@@ -110,3 +110,10 @@ osi_PostPopulateVCache(struct vcache *avc) {
    vSetType(avc, VNON);
 #endif
 }
+
+int
+osi_vnhold(struct vcache *avc)
+{
+    VN_HOLD(AFSTOV(avc));
+    return 0;
+}
