@@ -13,38 +13,62 @@
 #include "afs/sysincludes.h"	/*Standard vendor system headers */
 #include "afsincludes.h"	/*AFS-based standard headers */
 
-#if defined(AFS_FBSD80_ENV)
-#define ma_vn_lock(vp, flags, p) (vn_lock(vp, flags))
-#define MA_VOP_LOCK(vp, flags, p) (VOP_LOCK(vp, flags))
-#define MA_VOP_UNLOCK(vp, flags, p) (VOP_UNLOCK(vp, flags))
-#else
-#define ma_vn_lock(vp, flags, p) (vn_lock(vp, flags, p))
-#define MA_VOP_LOCK(vp, flags, p) (VOP_LOCK(vp, flags, p))
-#define MA_VOP_UNLOCK(vp, flags, p) (VOP_UNLOCK(vp, flags, p))
-#endif
-
 int
-osi_TryEvictVCache(struct vcache *avc, int *slept, int defersleep) {
+osi_TryEvictVCache(struct vcache *avc, int *slept, int defersleep)
+{
+    struct vnode *vp;
+    int code;
+    int evicted = 0;
 
-    /*
-     * essentially all we want to do here is check that the
-     * vcache is not in use, then call vgone() (which will call
-     * inactive and reclaim as needed).  This requires some
-     * kind of complicated locking, which we already need to implement
-     * for FlushVCache, so just call that routine here and check
-     * its return value for whether the vcache was evict-able.
-     */
-    if (osi_VM_FlushVCache(avc, slept) != 0)
-	return 0;
-    else
-	return 1;
+    vp = AFSTOV(avc);
+
+    if (!VI_TRYLOCK(vp))
+	return evicted;
+    code = osi_fbsd_checkinuse(avc);
+    if (code != 0) {
+	VI_UNLOCK(vp);
+	return evicted;
+    }
+
+    if ((vp->v_iflag & VI_DOOMED) != 0) {
+	VI_UNLOCK(vp);
+	evicted = 1;
+	return evicted;
+    }
+
+    vholdl(vp);
+
+    ReleaseWriteLock(&afs_xvcache);
+    AFS_GUNLOCK();
+
+    *slept = 1;
+
+    if (vn_lock(vp, LK_INTERLOCK|LK_EXCLUSIVE|LK_NOWAIT) == 0) {
+	/*
+	 * vrecycle() will vgone() only if its usecount is 0. If someone grabbed a
+	 * new usecount ref just now, the vgone() will be skipped, and vrecycle
+	 * will return 0.
+	 */
+	if (vrecycle(vp) != 0) {
+	    evicted = 1;
+	}
+
+	VOP_UNLOCK(vp, 0);
+    }
+
+    vdrop(vp);
+
+    AFS_GLOCK();
+    ObtainWriteLock(&afs_xvcache, 340);
+
+    return evicted;
 }
 
 struct vcache *
 osi_NewVnode(void) {
     struct vcache *tvc;
 
-    tvc = (struct vcache *)afs_osi_Alloc(sizeof(struct vcache));
+    tvc = afs_osi_Alloc(sizeof(struct vcache));
     tvc->v = NULL; /* important to clean this, or use memset 0 */
 
     return tvc;
@@ -58,24 +82,17 @@ osi_PrePopulateVCache(struct vcache *avc) {
 void
 osi_AttachVnode(struct vcache *avc, int seq) {
     struct vnode *vp;
-    struct thread *p = curthread;
 
     ReleaseWriteLock(&afs_xvcache);
     AFS_GUNLOCK();
-#if defined(AFS_FBSD60_ENV)
     if (getnewvnode(MOUNT_AFS, afs_globalVFS, &afs_vnodeops, &vp))
-#else
-    if (getnewvnode(MOUNT_AFS, afs_globalVFS, afs_vnodeop_p, &vp))
-#endif
 	panic("afs getnewvnode");	/* can't happen */
-#ifdef AFS_FBSD70_ENV
     /* XXX verified on 80--TODO check on 7x */
     if (!vp->v_mount) {
-        ma_vn_lock(vp, LK_EXCLUSIVE | LK_RETRY, p); /* !glocked */
+        vn_lock(vp, LK_EXCLUSIVE | LK_RETRY); /* !glocked */
         insmntque(vp, afs_globalVFS);
-        MA_VOP_UNLOCK(vp, 0, p);
+        VOP_UNLOCK(vp, 0);
     }
-#endif
     AFS_GLOCK();
     ObtainWriteLock(&afs_xvcache,339);
     if (avc->v != NULL) {
@@ -102,3 +119,19 @@ osi_PostPopulateVCache(struct vcache *avc) {
     vSetType(avc, VREG);
 }
 
+int
+osi_vnhold(struct vcache *avc)
+{
+    struct vnode *vp = AFSTOV(avc);
+
+    vref(vp);
+    VI_LOCK(vp);
+    if ((vp->v_iflag & VI_DOOMED) != 0) {
+	VI_UNLOCK(vp);
+	vrele(vp);
+	return ENOENT;
+    }
+
+    VI_UNLOCK(vp);
+    return 0;
+}
