@@ -48,17 +48,10 @@
  * check the VCS history of those files.
  */
 
-#if defined(AFS_FBSD80_ENV)
 #define	lock_vnode(v, f)	vn_lock((v), (f))
 #define ilock_vnode(v)	vn_lock((v), LK_INTERLOCK|LK_EXCLUSIVE|LK_RETRY)
 #define unlock_vnode(v)	VOP_UNLOCK((v), 0)
 #define islocked_vnode(v)	VOP_ISLOCKED((v))
-#else
-#define	lock_vnode(v, f)	vn_lock((v), (f), curthread)
-#define ilock_vnode(v)	vn_lock((v), LK_INTERLOCK|LK_EXCLUSIVE|LK_RETRY, curthread)
-#define unlock_vnode(v)	VOP_UNLOCK((v), 0, curthread)
-#define islocked_vnode(v)	VOP_ISLOCKED((v), curthread)
-#endif
 
 #if __FreeBSD_version >= 1000030
 #define AFS_VM_OBJECT_WLOCK(o)	VM_OBJECT_WLOCK(o)
@@ -86,8 +79,7 @@ osi_VM_FlushVCache(struct vcache *avc)
 
     vp = AFSTOV(avc);
 
-    if (!VI_TRYLOCK(vp))
-	return EBUSY;
+    VI_LOCK(vp);
     code = osi_fbsd_checkinuse(avc);
     if (code) {
 	VI_UNLOCK(vp);
@@ -180,6 +172,8 @@ osi_VM_TryToSmush(struct vcache *avc, afs_ucred_t *acred, int sync)
     }
     VI_UNLOCK(vp);
 
+    AFS_GUNLOCK();
+
     islocked = islocked_vnode(vp);
     if (islocked == LK_EXCLOTHER)
 	panic("Trying to Smush over someone else's lock");
@@ -220,6 +214,8 @@ osi_VM_TryToSmush(struct vcache *avc, afs_ucred_t *acred, int sync)
 	lock_vnode(vp, LK_DOWNGRADE);
     else if (!islocked)
 	unlock_vnode(vp);
+
+    AFS_GLOCK();
 }
 
 /* Purge VM for a file when its callback is revoked.
