@@ -23,35 +23,19 @@
 
 #define di_vicep1         di_un . di_icom .ic_gen
 #define di_vicep2         di_un . di_icom .ic_flags
-#if	defined(AFS_SUN56_ENV)
 #define di_vicep3       di_ic.ic_uid
 #define di_vicemagic    di_ic.ic_gid
-#else
-#define di_vicep3         di_un . di_icom .ic_size.val[0]
-#endif
 
 #define i_vicep1 i_ic.ic_gen
 #define i_vicep2 i_ic.ic_flags
-#if	defined(AFS_SUN56_ENV)
 #define i_vicep3 	i_ic.ic_uid
 #define i_vicemagic	i_ic.ic_gid
-#else
-#define i_vicep3 i_ic.ic_size.val[0]
-#endif
 
-#if	defined(AFS_SUN56_ENV)
 #define	IS_VICEMAGIC(ip)	((ip)->i_vicemagic == VICEMAGIC)
 #define	IS_DVICEMAGIC(dp)	((dp)->di_vicemagic == VICEMAGIC)
 
 #define	CLEAR_VICEMAGIC(ip)	(ip)->i_vicemagic = (ip)->i_vicep3 = 0
 #define	CLEAR_DVICEMAGIC(dp)	(dp)->di_vicemagic = (dp)->di_vicep3 = 0
-#else
-#define  IS_VICEMAGIC(ip)        (((ip)->i_vicep2 || (ip)->i_vicep3) ? 1 : 0)
-#define  IS_DVICEMAGIC(dp)       (((dp)->di_vicep2 || (dp)->di_vicep3) ? 1 : 0)
-
-#define  CLEAR_VICEMAGIC(ip)     (ip)->i_vicep2 = (ip)->i_vicep3 = 0
-#define  CLEAR_DVICEMAGIC(dp)    (dp)->di_vicep2 = (dp)->di_vicep3 = 0
-#endif
 
 #define AFS_SUN_UFS_CACHE 0
 #ifdef AFS_HAVE_VXFS
