@@ -64,11 +64,10 @@ int afs_iauth_initd = 0;
 
 extern int afs_NFSRootOnly;	/* 1 => only allow NFS mounts of /afs. */
 
-#if !defined(AFS_ATHENA_ENV)
 int
 #ifdef AFS_AIX41_ENV
 afs_fid(OSI_VC_DECL(avc), struct fid *fidpp, struct ucred *credp)
-#elif defined(AFS_SUN54_ENV)
+#elif defined(AFS_SUN5_ENV)
 afs_fid(OSI_VC_DECL(avc), struct fid *fidpp)
 #else
 afs_fid(OSI_VC_DECL(avc), struct fid **fidpp)
@@ -105,7 +104,7 @@ afs_fid(OSI_VC_DECL(avc), struct fid **fidpp)
 	if (avc->f.fid.Fid.Unique > 0xffffff)
 	    afs_fid_uniqueoverflow++;
     } else {
-#if defined(AFS_SUN57_64BIT_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZPTR == 64))
+#if defined(AFS_SUN5_64BIT_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZPTR == 64))
 	addr[1] = (long)AFS_XLATOR_MAGIC << 48;
 #else /* defined(AFS_SGI61_ENV) && (_MIPS_SZPTR == 64) */
 	addr[1] = AFS_XLATOR_MAGIC;
@@ -114,10 +113,10 @@ afs_fid(OSI_VC_DECL(avc), struct fid **fidpp)
 	addr[0] = (long)avc;
 #ifndef AFS_AIX41_ENV
 	/* No post processing, so don't hold ref count. */
-	AFS_FAST_HOLD(avc);
+	osi_Assert(osi_vnhold(avc) == 0);
 #endif
     }
-#if	defined(AFS_AIX_ENV) || defined(AFS_SUN54_ENV)
+#if	defined(AFS_AIX_ENV) || defined(AFS_SUN5_ENV)
     /* Use the fid pointer passed to us. */
     fidpp->fid_len = SizeOfSmallFid;
 
@@ -150,7 +149,6 @@ afs_fid(OSI_VC_DECL(avc), struct fid **fidpp)
 #endif
     return (0);
 }
-#endif
 
 
 #endif /* !AFS_LINUX20_ENV */
