@@ -194,9 +194,6 @@ afs_Daemon(void)
 	    afs_FlushReclaimedVcaches();
 	    ReleaseWriteLock(&afs_xvcache);
 	    afs_FlushActiveVcaches(1);	/* keep flocks held & flush nfs writes */
-#if 0
-	    afs_StoreDirtyVcaches();
-#endif
 	    last1MinCheck = now;
 	}
 
@@ -669,7 +666,7 @@ afs_BQueue(short aopcode, struct vcache *avc,
 		crhold(tb->cred);
 	    }
 	    if (avc) {
-		AFS_FAST_HOLD(avc);
+		osi_Assert(osi_vnhold(avc) == 0);
 	    }
 	    tb->refCount = ause + 1;
 	    tb->size_parm[0] = asparm0;
@@ -1064,7 +1061,9 @@ afs_BackgroundDaemon(void)
 	    ReleaseWriteLock(&afs_xbrs);
 	    afs_osi_Wakeup(&afs_termState);
 #ifdef AFS_NEW_BKG
-	    return -2;
+	    memset(uspc, 0, sizeof(*uspc));
+	    uspc->reqtype = AFS_USPC_SHUTDOWN;
+	    return 0;
 #else
 	    return;
 #endif
