@@ -94,6 +94,9 @@
 #define SYS_NAME_ID_x86_darwin_190       541
 #define SYS_NAME_ID_amd64_darwin_190     542
 #define SYS_NAME_ID_arm_darwin_190       543
+#define SYS_NAME_ID_x86_darwin_200       544
+#define SYS_NAME_ID_amd64_darwin_200     545
+#define SYS_NAME_ID_arm_darwin_200       546
 
 #define SYS_NAME_ID_next_mach20		 601
 #define SYS_NAME_ID_next_mach30		 602
@@ -240,7 +243,11 @@
 #define SYS_NAME_ID_i386_fbsd_104      2154
 #define SYS_NAME_ID_i386_fbsd_110      2140
 #define SYS_NAME_ID_i386_fbsd_111      2141
+#define SYS_NAME_ID_i386_fbsd_112      2142
+#define SYS_NAME_ID_i386_fbsd_113      2143
 #define SYS_NAME_ID_i386_fbsd_120      2160
+#define SYS_NAME_ID_i386_fbsd_121      2161
+#define SYS_NAME_ID_i386_fbsd_122      2162
 
 #define SYS_NAME_ID_ia64_linux2		2200
 #define SYS_NAME_ID_ia64_linux22	2201
@@ -353,7 +360,11 @@
 #define SYS_NAME_ID_amd64_fbsd_104     3054
 #define SYS_NAME_ID_amd64_fbsd_110     3040
 #define SYS_NAME_ID_amd64_fbsd_111     3041
+#define SYS_NAME_ID_amd64_fbsd_112     3042
+#define SYS_NAME_ID_amd64_fbsd_113     3043
 #define SYS_NAME_ID_amd64_fbsd_120     3060
+#define SYS_NAME_ID_amd64_fbsd_121     3061
+#define SYS_NAME_ID_amd64_fbsd_122     3062
 
 #define SYS_NAME_ID_amd64_w2k           3400
 
