@@ -3,7 +3,6 @@
 #ifndef	AFS_PARAM_COMMON_H
 #define	AFS_PARAM_COMMON_H 1
 
-#define AFS_64BIT_ENV  1
 #define AFS_NAMEI_ENV  1	/* User space interface to file system */
 #define AFS_64BIT_IOPS_ENV 1	/* Needed for NAMEI */
 #define AFS_64BIT_CLIENT 1
@@ -11,14 +10,6 @@
 #define AFS_MOUNT_AFS "afs"	/* The name of the filesystem type. */
 #define AFS_SYSCALL 318 /* 210 */
 
-#if 0
-/* including this file before sysincludes.h is canonical, but
- * NBSD40's mount.h defines MOUNT_AFS */
-#ifndef	MOUNT_AFS
-#define	MOUNT_AFS AFS_MOUNT_AFS
-#endif
-#endif
-
 #define AFS_XBSD_ENV 1		/* {Free,Open,Net}BSD */
 
 #define AFS_NBSD_ENV 1
@@ -56,7 +47,6 @@
 
 #include <afs/afs_sysnames.h>
 
-#define AFS_VM_RDWR_ENV	1
 #define AFS_VFS_ENV	1
 #define AFS_GREEDY43_ENV	1
 
@@ -66,7 +56,6 @@
 /* Extra kernel definitions (from kdefs file) */
 #ifdef _KERNEL
 #define AFS_GLOBAL_SUNLOCK        1
-#define	AFS_SHORTGID	1	/* are group id's short? */
 #endif
 
 /* apparently not in 40 */
@@ -75,7 +64,6 @@
 #ifdef _KERNEL_DEPRECATED
 #define AFS_GLOBAL_SUNLOCK        1
 #define	AFS_VFS34	1	/* What is VFS34??? */
-#define	AFS_SHORTGID	1	/* are group id's short? */
 #define	afsio_iov	uio_iov
 #define	afsio_iovcnt	uio_iovcnt
 #define	afsio_offset	uio_offset
