@@ -3,7 +3,6 @@
 #ifndef	AFS_PARAM_COMMON_H
 #define	AFS_PARAM_COMMON_H 1
 
-#define AFS_64BIT_ENV  1
 #define AFS_NAMEI_ENV  1	/* User space interface to file system */
 #define AFS_64BIT_IOPS_ENV 1	/* Needed for NAMEI */
 #define AFS_64BIT_CLIENT 1
@@ -11,26 +10,10 @@
 #define AFS_MOUNT_AFS "afs"	/* The name of the filesystem type. */
 #define AFS_SYSCALL 210
 
-
-#ifdef AFS_KALLOC
-#undef AFS_KALLOC
-#endif
-#define AFS_KALLOC(s) (osi_nbsd_Alloc((s), 1 /* cansleep */))
-
-#ifdef AFS_KFREE
-#undef AFS_KFREE
-#endif
-#define AFS_KFREE(p, s) (osi_nbsd_Free((p), (s)))
-
-#if 0
-/* including this file before sysincludes.h is canonical, but
- * NBSD40's mount.h defines MOUNT_AFS */
-
-#ifndef	MOUNT_AFS
-#define	MOUNT_AFS AFS_MOUNT_AFS
-#endif
-
-#endif /* 0 */
+#define AFS_KALLOC(n)           kmem_alloc(n, KM_SLEEP)
+#define AFS_KALLOC_NOSLEEP(n)   kmem_alloc(n, KM_NOSLEEP)
+#define AFS_KFREE               kmem_free
+#define VATTR_NULL              vattr_null
 
 #define AFS_XBSD_ENV 1		/* {Free,Open,Net}BSD */
 
@@ -41,6 +24,7 @@
 #define AFS_NBSD30_ENV 1
 #define AFS_NBSD40_ENV 1
 #define AFS_NBSD50_ENV 1
+#define AFS_NBSD60_ENV 1
 #undef  AFS_NONFSTRANS
 #define AFS_NONFSTRANS 1
 
@@ -69,7 +53,6 @@
 
 #include <afs/afs_sysnames.h>
 
-#define AFS_VM_RDWR_ENV	1
 #define AFS_VFS_ENV	1
 #define AFS_GREEDY43_ENV	1
 
@@ -80,7 +63,6 @@
 /* Extra kernel definitions (from kdefs file) */
 #ifdef _KERNEL_DEPRECATED
 #define	AFS_VFS34	1	/* What is VFS34??? */
-#define	AFS_SHORTGID	1	/* are group id's short? */
 #define	afsio_iov	uio_iov
 #define	afsio_iovcnt	uio_iovcnt
 #define	afsio_offset	uio_offset
@@ -158,7 +140,7 @@ enum vcexcl { NONEXCL, EXCL };
 #define CMSERVERPREF
 #endif
 
-#if	!defined(ASSEMBLER) && !defined(__LANGUAGE_ASSEMBLY__) && !defined(IGNORE_STDS_H)
+#if	!defined(ASSEMBLER) && !defined(__LANGUAGE_ASSEMBLY__) && !defined(IGNORE_STDS_H) && !defined()
 #include <limits.h>
 #include <sys/param.h>
 #include <sys/types.h>
