@@ -37,7 +37,6 @@
 
 #define _ANSI_C_SOURCE		1	/* rx_user.h */
 
-#define AFS_64BIT_ENV		1	/* Defines afs_int32 as int, not long. */
 #define AFS_64BIT_CLIENT	1
 #define AFS_64BITPOINTER_ENV	1	/* pointers are 64 bits. */
 #define AFS_64BITUSERPOINTER_ENV	1
@@ -63,16 +62,6 @@
 #define AFS_SGI_XFS_IOPS_ENV 	1	/* turns on XFS inode ops. */
 #define AFS_64BIT_IOPS_ENV	1	/* inode ops expect 64 bit inodes */
 
-/* Vnode size and access differs between Octane and Origin. The number
- * can be used to indicate which altername vnodeX_t to use for future
- * changes.
- */
-#ifdef notdef
-/* SGI may have cleared this problem up. */
-#define AFS_SGI_VNODE_GLUE	1
-#endif
-
-
 /* File system entry (used if mount.h doesn't define MOUNT_AFS */
 #define AFS_MOUNT_AFS	 "afs"
 
@@ -177,8 +166,6 @@
 #define AFS_HAVE_FFS            1	/* Use system's ffs. */
 #define AFS_HAVE_STATVFS	1	/* System supports statvfs */
 
-#define AFS_64BIT_ENV		1	/* Defines afs_int32 as int, not long. */
-
 #include <afs/afs_sysnames.h>
 
 /* Extra kernel definitions (from kdefs file) */
