@@ -12,19 +12,10 @@
 #define AFS_ENV			1
 #define AFS_SUN_ENV		1
 #define AFS_SUN5_ENV		1
-#define	AFS_SUN52_ENV		1
-#define	AFS_SUN53_ENV		1
-#define	AFS_SUN54_ENV		1
-#define	AFS_SUN55_ENV		1
-#define	AFS_SUN56_ENV		1
-#define AFS_SUN57_ENV		1
-#define AFS_SUN58_ENV		1
 #define AFS_SUN59_ENV		1
 #define AFS_SUN510_ENV		1
 #define AFS_SUN511_ENV		1
-#define AFS_BOZONLOCK_ENV       1
 
-#define AFS_64BIT_ENV		1	/* Defines afs_int32 as int, not long. */
 #define AFS_64BIT_CLIENT	1
 
 #define AFS_HAVE_FLOCK_SYSID	1
@@ -91,10 +82,9 @@
 /**
   * These defines are for the 64 bit Solaris 7 port
   * AFS_SYSCALL32 is used to protect the ILP32 syscall interface
-  * AFS_64BIT_ENV is for use of 64 bit inode numbers
   */
 #if defined(__sparcv9)
-#define	AFS_SUN57_64BIT_ENV	1
+#define	AFS_SUN5_64BIT_ENV	1
 #define AFS_64BIT_INO   	1
 #endif
 
@@ -133,14 +123,9 @@
 #define AFS_USR_SUN8_ENV	1
 #define AFS_USR_SUN9_ENV	1
 
-#define AFS_64BIT_ENV		1
 
 #include <afs/afs_sysnames.h>
 
-#if 0
-#define AFS_GLOBAL_SUNLOCK	1	/* For global locking */
-#endif
-
 #define	AFS_3DISPARES		1	/* Utilize the 3 available disk inode 'spares' */
 
 /* so we get _IOW() when we include sys/ioctl.h */
