@@ -467,7 +467,7 @@ main(int argc, char *argv[])
 	if (file == NULL)
 	    usage();
 	len = filelength(_fileno(file));
-	save = (char *)malloc(len + 1);
+	save = malloc(len + 1);
 	buf = save;
 	len = fread(buf, sizeof(char), len, file);
 	buf[len] = 0;		//set eof
@@ -633,7 +633,7 @@ main(int argc, char *argv[])
 	if (file == NULL)
 	    exit(0xc000);
 	len = filelength(_fileno(file));
-	save = (char *)malloc(len + 2);
+	save = malloc(len + 2);
 	ch = save;
 	*ch++ = 0;		/* a small hack to allow matching /r/n if /n is first character */
 	len = fread(ch, sizeof(char), len, file);
@@ -655,7 +655,7 @@ main(int argc, char *argv[])
 	if (file == NULL)
 	    exit(0xc000);
 	len = filelength(_fileno(file));
-	save = (char *)malloc(len + 1);
+	save = malloc(len + 1);
 	ch = save;
 	len = fread(ch, sizeof(char), len, file);
 	file = freopen(fname, "wb", file);
@@ -674,7 +674,7 @@ main(int argc, char *argv[])
     if (file == NULL)
 	exit(0xc000);
     len = filelength(_fileno(file));
-    save = (char *)malloc(len + 1);
+    save = malloc(len + 1);
     ch = save;
     len = fread(ch, sizeof(char), len, file);
     file = freopen(fname, "wb", file);
@@ -684,7 +684,7 @@ main(int argc, char *argv[])
 	ch++;
     }
     fclose(file);
-    pvar = (char **)malloc(argc * sizeof(char *));
+    pvar = malloc(argc * sizeof(char *));
     for (i = 1; i < argc - 1; i++)
 	pvar[i] = argv[i + 1];
     pvar[argc - 1] = NULL;
