@@ -18,18 +18,11 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-
-#include <sys/types.h>
-#include <sys/file.h>
-#if !defined(AFS_SUN5_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD80_ENV)
+#if !defined(AFS_SUN5_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_FBSD_ENV)
 #include <sgtty.h>
 #endif
-#include <stdio.h>
-#include <sys/time.h>
-#include <errno.h>
-#include <string.h>
-#include <stdlib.h>
 
 #include <lwp.h>
 
@@ -37,8 +30,6 @@
 #include "gtxobjects.h"
 #include "gtxframe.h"
 
-
-
 int curses_debug;		/*Is debugging turned on? */
 static char mn[] = "gator_curseswindows";	/*Module name */
 
@@ -116,8 +107,7 @@ gator_cursesgwin_init(int adebug)
 	fprintf(stderr,
 		"[%s:%s] Allocating %" AFS_SIZET_FMT " bytes for curses window private space in base window\n",
 		mn, rn, sizeof(struct gator_cursesgwin));
-    c_data =
-	(struct gator_cursesgwin *)malloc(sizeof(struct gator_cursesgwin));
+    c_data = malloc(sizeof(struct gator_cursesgwin));
     if (c_data == (struct gator_cursesgwin *)0) {
 	fprintf(stderr,
 		"[%s:%s] Can't allocate %" AFS_SIZET_FMT " bytes for curses window private space in base window\n",
@@ -200,7 +190,7 @@ gator_cursesgwin_create(void * rock)
 	fprintf(stderr,
 		"[%s:%s] Allocating %" AFS_SIZET_FMT " bytes for new gwin structure\n", mn,
 		rn, sizeof(struct gwin));
-    newgwin = (struct gwin *)malloc(sizeof(struct gwin));
+    newgwin = malloc(sizeof(struct gwin));
     if (newgwin == NULL) {
 	fprintf(stderr,
 		"[%s:%s] Can't malloc() %" AFS_SIZET_FMT " bytes for new gwin structure: Errno is %d\n",
@@ -221,8 +211,7 @@ gator_cursesgwin_create(void * rock)
 	fprintf(stderr,
 		"[%s:%s] Allocating %" AFS_SIZET_FMT " bytes for curses window private space\n",
 		mn, rn, sizeof(struct gator_cursesgwin));
-    c_data =
-	(struct gator_cursesgwin *)malloc(sizeof(struct gator_cursesgwin));
+    c_data = malloc(sizeof(struct gator_cursesgwin));
     if (c_data == (struct gator_cursesgwin *)0) {
 	fprintf(stderr,
 		"[%s:%s] Can't allocate %" AFS_SIZET_FMT " bytes for curses window private space\n",
@@ -582,15 +571,17 @@ gator_cursesgwin_drawchar(struct gwin *gwp, struct gwin_charparams *params)
 		mn, rn, params->c, cwp->wp, params->x, params->y, curses_y,
 		curses_x, (params->highlight ? ", using standout mode" : ""));
     wmove(cwp->wp, curses_y, curses_x);
-    if (params->highlight)
+    if (params->highlight) {
 	code=wstandout(cwp->wp);
         if (code)
 	    return (code);
+    }
     waddch(cwp->wp, params->c);
-    if (params->highlight)
+    if (params->highlight) {
 	code=wstandend(cwp->wp);
         if (code)
             return (code);
+    }
 
     return (0);
 
@@ -636,15 +627,17 @@ gator_cursesgwin_drawstring(struct gwin *gwp, struct gwin_strparams *params)
 		mn, rn, params->s, cwp->wp, params->x, params->y, curses_y,
 		curses_x, (params->highlight ? ", using standout mode" : ""));
     wmove(cwp->wp, curses_y, curses_x);
-    if (params->highlight)
+    if (params->highlight) {
 	code=wstandout(cwp->wp);
 	if (code)
 	    return (code);
+    }
     waddstr(cwp->wp, params->s);
-    if (params->highlight)
+    if (params->highlight) {
 	code=wstandend(cwp->wp);
 	if (code)
 	    return (code);
+    }
 
     return (code);
 
