@@ -27,7 +27,6 @@
 #include "../rx/rx_kernel.h"	/* For osi_Panic() */
 
 #define RX_ENABLE_LOCKS         1
-#define AFS_GLOBAL_RXLOCK_KERNEL
 
 extern lock_t *rx_sleepLock;
 
@@ -74,48 +73,6 @@ extern lock_t *rx_sleepLock;
 	} while(0)
 
 
-#if 0
-#define CV_WAIT(cv, lck) \
-    do { \
-        int code; \
-        ksleep_prepare(); \
-        MP_SPINLOCK(rx_sleepLock); \
-        if (!b_owns_sema(lck)) \
-            osi_Panic("mutex not held \n"); \
-        b_vsema(lck); \
-        code = ksleep_one(PCATCH | KERNEL_ADDRESS | KERN_SPINLOCK_OBJECT, \
-            (cv), rx_sleepLock, 0); \
-        if (code) { \
-            if (code == EINTR) { /* lock still held */ \
-                MP_SPINUNLOCK(rx_sleepLock); \
-            } else if (code != -EINTR) { \
-                osi_Panic("ksleep_one failed: code = %d \n", code); \
-            } \
-        } \
-        b_psema(lck); /* grab the mutex again */ \
-    } while(0)
-
-/* Wakes up a thread waiting on this condition */
-#define CV_SIGNAL(cv) \
-    do { \
-        int wo, code; \
-        MP_SPINLOCK(rx_sleepLock); \
-        if ((code = kwakeup_one(KERNEL_ADDRESS, (cv), WAKEUP_ONE, &wo)) < 0) \
-            osi_Panic("kwakeup_one failed: code = %d \n", code); \
-        MP_SPINUNLOCK(rx_sleepLock); \
-    } while (0)
-
-/* Wakes up all threads waiting on this condition */
-#define CV_BROADCAST(cv) \
-    do { \
-        int wo, code; \
-        MP_SPINLOCK(rx_sleepLock); \
-        if ((code = kwakeup_one(KERNEL_ADDRESS, (cv), WAKEUP_ALL, &wo)) < 0) \
-            osi_Panic("kwakeup_all failed: code = %d \n", code); \
-        MP_SPINUNLOCK(rx_sleepLock); \
-    } while (0)
-#endif /* 0 */
-
 #define CV_DESTROY(a)
 
 /* We now use beta semaphores for mutexes */
@@ -133,8 +90,6 @@ typedef caddr_t afs_kcondvar_t;
 #ifdef AFS_HPUX102_ENV
 
 #if defined(AFS_HPUX110_ENV)
-#undef osirx_AssertMine
-extern void osirx_AssertMine(afs_kmutex_t * lockaddr, char *msg);
 
 #define AFS_RX_ORDER 30
 
@@ -156,18 +111,16 @@ extern void osirx_AssertMine(afs_kmutex_t * lockaddr, char *msg);
     ((b_owns_sema(a)) ? b_vsema(a) : (osi_Panic("mutex not held"), 0))
 #endif
 
-#undef MUTEX_ISMINE
-#define MUTEX_ISMINE(a) b_owns_sema(a)
+#define MUTEX_ASSERT(a) osi_Assert(b_owns_sema(a))
 
 #else /* AFS_HPUX110_ENV */
 
-#define osirx_AssertMine(addr, msg)
-
 #define MUTEX_DESTROY(a)
 #define MUTEX_ENTER(a)
 #define MUTEX_TRYENTER(a) 1
 #define MUTEX_EXIT(a)
 #define MUTEX_INIT(a,b,c,d)
+#define MUTEX_ASSERT(a)
 
 #endif /* else AFS_HPUX110_ENV */
 #endif /* AFS_HPUX102_ENV */
