@@ -86,6 +86,11 @@ extern int clock_nUpdates;
 
 /* Current clock time, truncated to seconds */
 #define	clock_Sec() ((!clock_haveCurrentTime)? clock_UpdateTime(), clock_now.sec:clock_now.sec)
+
+extern void clock_Init(void);
+extern int clock_UnInit(void);
+extern void clock_UpdateTime(void);
+
 #endif /* AFS_USE_GETTIMEOFDAY || AFS_PTHREAD_ENV */
 #else /* KERNEL */
 #define clock_Init()
@@ -185,7 +190,7 @@ extern int clock_nUpdates;
                      + 2 * ((c2)->usec / 1000) * ((c2)->usec % 1000) / 1000   \
                      + ((((c2)->usec % 1000) > 707) ? 1 : 0);                 \
      }                                                                        \
-   if ((c1)->usec > 1000000) {                                                \
+   while ((c1)->usec > 1000000) {                                             \
         (c1)->usec -= 1000000;                                                \
         (c1)->sec++;                                                          \
    }                                                                          \
