@@ -42,10 +42,10 @@ typedef unsigned short etap_event_t;
 #include "h/systm.h"
 #endif
 #include "h/time.h"
-#ifdef AFS_SUN56_ENV
-#include "h/vfs.h"		/* stops SUN56 socketvar.h warnings */
-#include "h/stropts.h"		/* stops SUN56 socketvar.h warnings */
-#include "h/stream.h"		/* stops SUN56 socketvar.h errors */
+#ifdef AFS_SUN5_ENV
+#include "h/vfs.h"		/* stops SUN5 socketvar.h warnings */
+#include "h/stropts.h"		/* stops SUN5 socketvar.h warnings */
+#include "h/stream.h"		/* stops SUN5 socketvar.h errors */
 #include "h/disp.h"
 #endif
 #include "h/socket.h"
@@ -133,9 +133,6 @@ typedef unsigned short etap_event_t;
 #if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX22_ENV) && !defined(AFS_DARWIN_ENV)
 #include "netinet/in_var.h"
 #endif /* ! AFS_HPUX110_ENV && ! AFS_LINUX22_ENV */
-#if !defined(AFS_LINUX22_ENV) && !defined(AFS_DUX40_ENV)
-#include "rpc/types.h"
-#endif
 #include "afs/afs_osi.h"
 #include "rx_kmutex.h"
 #include "afs/lock.h"
@@ -149,7 +146,11 @@ typedef unsigned short etap_event_t;
 # if defined(AFS_FBSD_ENV)
 #  include "machine/stdarg.h"
 # else
-#  include "stdarg.h"
+#  if defined(HAVE_LINUX_STDARG_H)
+#   include "linux/stdarg.h"
+#  else
+#   include "stdarg.h"
+#  endif
 # endif
 #endif
 #ifdef KERNEL
