@@ -104,11 +104,11 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
 
 #if !defined(AFS_NT40_ENV)
     if (ntohs(port) >= IPPORT_RESERVED && ntohs(port) < IPPORT_USERRESERVED) {
-/*	(osi_Msg "%s*WARNING* port number %d is not a reserved port number.  Use port numbers above %d\n", name, port, IPPORT_USERRESERVED);
+/*	osi_Msg("%s*WARNING* port number %d is not a reserved port number.  Use port numbers above %d\n", name, port, IPPORT_USERRESERVED);
 */ ;
     }
     if (ntohs(port) > 0 && ntohs(port) < IPPORT_RESERVED && geteuid() != 0) {
-	(osi_Msg
+	osi_Msg(
 	 "%sport number %d is a reserved port number which may only be used by root.  Use port numbers above %d\n",
 	 name, ntohs(port), IPPORT_USERRESERVED);
 	goto error;
@@ -144,7 +144,7 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
         break;
     }
     if (code) {
-	(osi_Msg "%sbind failed\n", name);
+	osi_Msg("%sbind failed\n", name);
 	goto error;
     }
 #if !defined(AFS_NT40_ENV)
@@ -191,7 +191,7 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
 	     (socketFd, SOL_SOCKET, SO_RCVBUF, (char *)&len2,
 	      sizeof(len2)) >= 0);
 	if (!greedy)
-	    (osi_Msg "%s*WARNING* Unable to increase buffering on socket\n",
+	    osi_Msg("%s*WARNING* Unable to increase buffering on socket\n",
 	     name);
         if (rx_stats_active)
             rx_atomic_set(&rx_stats.socketGreedy, greedy);
@@ -230,13 +230,22 @@ rxi_GetUDPSocket(u_short port)
     return rxi_GetHostUDPSocket(htonl(INADDR_ANY), port);
 }
 
+void
+osi_Msg(const char *fmt, ...)
+{
+    va_list ap;
+    va_start(ap, fmt);
+    vfprintf(stderr, fmt, ap);
+    va_end(ap);
+}
+
 void
 osi_Panic(char *msg, ...)
 {
     va_list ap;
     va_start(ap, msg);
-    (osi_Msg "Fatal Rx error: ");
-    (osi_VMsg msg, ap);
+    fprintf(stderr, "Fatal Rx error: ");
+    vfprintf(stderr, msg, ap);
     va_end(ap);
     fflush(stderr);
     fflush(stdout);
@@ -352,7 +361,6 @@ rx_getAllAddrMaskMtu(afs_uint32 addrBuffer[], afs_uint32 maskBuffer[],
 #endif
 
 #ifdef AFS_NT40_ENV
-extern rx_atomic_t rxinit_status;
 void
 rxi_InitMorePackets(void) {
     int npackets, ncbufs;
@@ -373,7 +381,7 @@ rx_GetIFInfo(void)
 
     LOCK_IF_INIT;
     if (Inited) {
-	if (Inited < 2 && !rx_atomic_test_bit(&rxinit_status, 0)) {
+	if (Inited < 2 && rxi_IsRunning()) {
             /* We couldn't initialize more packets earlier.
              * Do it now. */
             rxi_InitMorePackets();
@@ -407,11 +415,11 @@ rx_GetIFInfo(void)
     UNLOCK_IF;
 
     /*
-     * If rxinit_status is still set, rx_InitHost() has yet to be called
+     * If rxi_IsRunning is false, rx_InitHost() has yet to be called
      * and we therefore do not have any mutex locks initialized.  As a
      * result we cannot call rxi_MorePackets() without crashing.
      */
-    if (rx_atomic_test_bit(&rxinit_status, 0))
+    if (!rxi_IsRunning())
         return;
 
     rxi_InitMorePackets();
@@ -782,21 +790,20 @@ rx_SetMaxMTU(int mtu)
 
 #ifdef AFS_RXERRQ_ENV
 int
-rxi_HandleSocketError(int socket)
+osi_HandleSocketError(int socket, void *cmsgbuf, size_t cmsgbuf_len)
 {
     struct msghdr msg;
     struct cmsghdr *cmsg;
     struct sock_extended_err *err;
     struct sockaddr_in addr;
-    char controlmsgbuf[256];
     int code;
 
     msg.msg_name = &addr;
     msg.msg_namelen = sizeof(addr);
     msg.msg_iov = NULL;
     msg.msg_iovlen = 0;
-    msg.msg_control = controlmsgbuf;
-    msg.msg_controllen = 256;
+    msg.msg_control = cmsgbuf;
+    msg.msg_controllen = cmsgbuf_len;
     msg.msg_flags = 0;
     code = recvmsg(socket, &msg, MSG_ERRQUEUE|MSG_DONTWAIT|MSG_TRUNC);
 
