@@ -12,21 +12,16 @@
  */
 
 #include <afsconfig.h>
-#include "afs/param.h"
-
+#include <afs/param.h>
 
 #if defined(KERNEL) && !defined(UKERNEL)
 #ifdef AFS_LINUX20_ENV
 #include "h/string.h"
-#if 0
-#define bzero(A, C) memset((A), 0, (C))
-#endif
 #else
 #include <sys/param.h>
 #include <sys/systm.h>
 #endif
-#else
-#include <stdio.h>
+#include <roken.h>
 #endif
 #include "xdr.h"
 
