@@ -16,17 +16,11 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-#include <signal.h>
+
+#include <roken.h>
 
 #include <afs/afs_args.h>
-#if defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)
-#include <unistd.h>
-#else
-#include <stdio.h>
-#endif
-#if defined(HAVE_UNISTD_H)
-#include <unistd.h>
-#endif
+
 #include "afssyscalls.h"
 #include "sys_prototypes.h"
 
@@ -57,9 +51,14 @@ lpioctl(char *path, int cmd, void *cmarg, int follow)
     rval = proc_afs_syscall(AFSCALL_PIOCTL, (long)path, cmd, (long)cmarg,
 			    follow, &errcode);
 
-    if(rval)
+    if(rval) {
+# ifdef AFS_SYSCALL
 	errcode = syscall(AFS_SYSCALL, AFSCALL_PIOCTL, path, cmd, cmarg,
 			  follow);
+# else
+	errcode = -1;
+# endif
+    }
 
     return (errcode);
 }
