@@ -18,24 +18,18 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
-#include "uss_fs.h"		/*Interface to this module */
-#include <sys/types.h>
-#include <errno.h>
-#include <sys/socket.h>
 #ifdef	AFS_SUN5_ENV
 #include <sys/ioccom.h>
 #endif
-#include <netinet/in.h>
-
-#include <string.h>
-#include <unistd.h>
 
 #include <afs/venus.h>
 #include <rx/rx.h>
 #include <afs/sys_prototypes.h>
-#include "uss_common.h"
 
+#include "uss_common.h"
+#include "uss_fs.h"		/*Interface to this module */
 
 /*
  * ---------------------- Private definitions ---------------------
@@ -565,8 +559,7 @@ uss_fs_UnlogToken(char *celln)
 	cnt++;
     } while (!code);
     count = cnt - 1;
-    tokenInfoP =
-	(struct tokenInfo *)malloc((sizeof(struct tokenInfo) * count));
+    tokenInfoP = malloc((sizeof(struct tokenInfo) * count));
     for (code = index = index2 = 0; (!code) && (index < count); index++) {
 	tp = tokenInfoP + index;
 	code = ktc_ListTokens(index2, &index2, &tp->service);
@@ -586,7 +579,7 @@ uss_fs_UnlogToken(char *celln)
 	       code);
 	exit(1);
     }
-    for (code = index = 0; index < count; index++) {
+    for (index = 0; index < count; index++) {
 	tp = tokenInfoP + index;
 	if (!(tp->deleted)) {
 	    code = ktc_SetToken(&tp->service, &tp->token, &tp->client, 0);
